//*****************************************************************************
//*
//*
//*		SizeSynch.cpp
//*
//*
//*****************************************************************************
//
//	Mit der Klasse CSizeSynch knnen zwei Pfade ber ein Pufferverzeichniss
//	synchronisiert werden. Dabei kann die maximale Gre des Verzeichnisses
//	angegeben werden. Reicht die Gre nicht aus, so wird nur ein Teil
//	synchronisiert.
//
#include	<FileFunctions.h>
#include	<BitFunctions.h>
#include	<StringChain.h>
#include	<FileFinder.h>
#include	<MapFile.h>
#include	<windows.h>
#include	<stdio.h>

#include	"SizeSynch.h"

#define  	SS_CALLBACK(m,p)	if(m_pCallback)m_iStop=m_pCallback(m_pCallParam,m,p);


typedef	struct
	{
	unsigned	uHost;									// Ist die Hostnummer
	unsigned	uTime;									// Letzte nderungszeit
	unsigned	uLength;								// Dateilnge
	unsigned	uChkSum;								// Checksummme
	}InfoTag;


char	CSizeSynch::m_cHostName[64];


//*****************************************************************************
//*
//*		EntryCmp
//*
//*****************************************************************************
//	Vergeleichsfunktion
static int __cdecl EntryCmp(CStringChain *pData,int iPos1,int iPos2)
{
const char	   *pName1;
const char	   *pName2;
InfoTag		   *pInfo1;
InfoTag		   *pInfo2;
int				iRet;


	pName1 = pData->Get(iPos1);
	pName2 = pData->Get(iPos2);
	iRet   = stricmp(pName1,pName2);

	if(iRet)return iRet;

	pInfo1 = (InfoTag*)pData->Data(iPos1);
	pInfo2 = (InfoTag*)pData->Data(iPos2);



return pInfo1->uHost-pInfo2->uHost;
}

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CSizeSynch::CSizeSynch()
{

	if(!m_cHostName[0])
		{
		WSADATA		sData;

		WSAStartup(2,&sData);
		gethostname(m_cHostName,sizeof(m_cHostName));
		}

	m_pCallback		= 0;
	m_pCallParam	= 0;
	m_iStop			= 0;
}

//*****************************************************************************
//*
//*		GetChecksum
//*
//*****************************************************************************
//	Berechnet die Checksumme einer Datei.
//	pFileName	: Ist der Name der Datei.
//	Ergibt die Checksumme, bzw. 0 bei einem Fehler.
unsigned CSizeSynch::GetChecksum(const char *pFileName)
{
unsigned       *pMem;
CMapFile		sFile;
unsigned		uSize;
unsigned		uSum;
unsigned		uMax;


	SS_CALLBACK(SSY_SCAN,pFileName);

		pMem = (unsigned*)sFile.Open(pFileName);
	if(!pMem)return 0;

	uSize	= sFile.GetLength();
	uMax	= uSize/sizeof(unsigned);
	uSum	= 0;

	while(uMax>0)
		{
		uSum += *pMem;
		bit_rol(uSum,5);
		pMem += 1;
		uMax -= 1;
		}

	uSize &= sizeof(unsigned)-1;

	if(uSize==1)uSum += *(unsigned char *)pMem;
	if(uSize==2)uSum += *(unsigned short*)pMem;
	if(uSize==3)
		{
		uSum += *(unsigned short*)pMem;
		uSum += ((unsigned char *)pMem)[2]<<16;
		}

	if(!uSum)uSum=1;



return uSum;
}

//*****************************************************************************
//*
//*		CreateList
//*
//*****************************************************************************
//	Erzeugt eine neue Liste mit den Quelldateien.
//	pFileList	: Ist der Name der Scnchronisationsliste (*.ssy)
//	pSource		: Ist der Pfad mit den Quelldateien
//	pPlus		: Ist der Plusfilter fr die Dateien
//	pMinus		: Ist der Minusfilter fr die Dateien
//	uSize		: Ist die maximale Gre
//	Ergibt die Anzahl der eingefgten Dateien bzw. -1 bei einem Fehler.
int CSizeSynch::CreateList(const char *pFileList,const char *pSource,const char *pPlus,const char *pMinus,unsigned uSize)
{
char			cPath[512];
char			cName[512];
char			cTemp[512];
FILE		   *pFile;
const char	   *pName;
InfoTag		   *pInfo;
InfoTag			sInfo;
CFileFinder		sFind;
CStringChain	sData;
int				iPos;



	SS_CALLBACK(SSY_MESSAGE,"Suche Dateien.");

	sFind.Read(pSource,CFF_FIND_SUBDIR,pPlus,pMinus);
	sData.DataLength(sizeof(sInfo));

	for(;;)												// Dateien suchen
		{
			pName = sFind.Next(1);
		if(!pName)break;

		sInfo.uTime		= sFind.Time();
		sInfo.uLength	= sFind.Len();
		sInfo.uChkSum	= GetChecksum(pName);

		if(!sInfo.uChkSum)continue;

		pName = sFind.Name(-1,2);
		sData.Add(pName,CSC_ABC,&sInfo);
		}


	SS_CALLBACK(SSY_MESSAGE,"Speichere die Liste.");

		pFile = fopen(pFileList,"wb");
	if(!pFile)return -1;



	if(!pPlus )pPlus ="";
	if(!pMinus)pMinus="";

	fprintf(pFile,"PLUS  \"%s\"\n",pPlus);
	fprintf(pFile,"MINUS \"%s\"\n",pMinus);
	fprintf(pFile,"HOST0 \"%s\"\n",m_cHostName);
	fprintf(pFile,"SIZE  %u\n"    ,uSize);


	for(iPos=0;iPos<sData.Count();iPos++)				// Dateiliste speichern
		{
		pName = sData.Get(iPos);
		pInfo = (InfoTag*)sData.Data(iPos);
		fprintf(pFile,"0\t0x%08X 0x%08X 0x%08X\t\"%s\"\n",pInfo->uLength,pInfo->uTime,pInfo->uChkSum,pName);
		}

	fclose(pFile);

	GetFileBody(cTemp,pFileList,sizeof(cTemp));
	sprintf(cName,"SsyTemp_%s\\",cTemp);

	GetFilePath(cPath,pFileList,sizeof(cPath));
	SetFileName(cPath,cName    ,sizeof(cPath));
	RemovePath (cPath);
	CreatePath (cPath);



return sData.Count();
}



//*****************************************************************************
//*
//*		Synch
//*
//*****************************************************************************
//	Starte die Synchronisation
//	pFileName	: Ist der Name der Datei.
//	pSource		: Ist der Pfad mit den Quelldateien bzw. Zieldateien
//	pHostCheck	: Wenn dieser Zeiger nicht 0 ist, wird der Hostnamen berprft
//				  und wenn ntig die Synchdatei neu erstellt.
//	iIsTarget	: Ist der Hostnamen ein Zielrechner, wenn ja wird
//				  ein Fehler geliefert wenn der Zielhost nicht in der Liste ist.
//	uSizeNew	: Ist die neue Puffergre. (Bei 0 keine nderung)
//	Ergibt die Anzahl der Synchronisierten Dateien bzw.
//					-1 = Bei einem Fehler
//					-2 = Wenn ein Zielhost nicht in der Liste ist
//					-3 = Bei einem User-Abbruch
int CSizeSynch::Synch(const char *pFileList,const char *pSource,const char *pHostCheck,int iIsTarget,unsigned uSizeNew)
{
const char     *pName;
FILE		   *pFile;
InfoTag		   *pNext;
InfoTag		   *pInfo;
InfoTag			sInfo;
CFileFinder		sFind;
CStringChain	sData;
unsigned		uSize;
unsigned		uByte;
unsigned 		uHost;
char			cBits [16];
char			cHost [64*10];
char			cMinus[512];
char			cPlus [512];
char			cTemp [512];
char			cBase [512];
char			cName [512];
char			cPath [256];
int				iNull;
int				iSave;
int				iLen;
int				iPos;
int				iNum;
int				iOff;
int				iRet;
int				iAt;
int				iOk;


	m_iStop   = 0;
	cMinus[0] = 0;
	cPlus [0] = 0;

	SS_CALLBACK(SSY_MESSAGE,"ffne die Liste.");

		pFile = fopen(pFileList,"rb");
	if(!pFile)return -1;

	GetFileBody(cTemp,pFileList,sizeof(cTemp));
	sprintf(cName,"SsyTemp_%s\\",cTemp);


	memset(cHost,0,sizeof(cHost));
	iLen = GetFilePath(cPath,pFileList,sizeof(cPath));
	iLen = SetFileName(cPath,cName    ,sizeof(cBase));
	iNum = strlcpy(cBase,pSource      ,sizeof(cBase));
	iNum = AddBackSlash(cBase,iNum);



	sData.DataLength(sizeof(sInfo));
	uSize = 100*1024*1024;
	iSave = 0;




	while(fgets(cTemp,sizeof(cTemp),pFile))				// Datei einlesen
		{
		iOff = strgetstr(cName,cTemp,sizeof(cName));

		SS_CALLBACK(SSY_STOP,cName);

		if(m_iStop)break;

		if(!strcmp (cName,"PLUS"  )){strgetstr(cPlus ,cTemp+iOff,sizeof(cPlus ));continue;}
		if(!strcmp (cName,"MINUS" )){strgetstr(cMinus,cTemp+iOff,sizeof(cMinus));continue;}
		if(!strcmp (cName,"SIZE"  )){uSize = strtoi(cTemp+iOff);                 continue;}
		if(!strncmp(cName,"HOST",4))
			{
			if(cName[4]<'0')continue;
			if(cName[4]>'9')continue;
			if(cName[5]!=0 )continue;

			strgetstr(cHost+(cName[4]-'0')*64,cTemp+iOff,64);
			continue;
			}

		if(cName[0]<'0')continue;
		if(cName[0]>'9')continue;
		if(cName[1]!=0 )continue;

		sInfo.uHost		= cName[0]-'0';

		iOff += strgetstr(cName,cTemp+iOff,sizeof(cName));

		sInfo.uLength	= strtoi(cName);

		iOff += strgetstr(cName,cTemp+iOff,sizeof(cName));

		sInfo.uTime		= strtoi(cName);

		iOff += strgetstr(cName,cTemp+iOff,sizeof(cName));

		sInfo.uChkSum	= strtoi(cName);

		iOff += strgetstr(cName,cTemp+iOff,sizeof(cName));

		sData.Add(cName,CSC_END,&sInfo);
		}

	sData.Sort(0,-1,EntryCmp);
	fclose(pFile);

	if(m_iStop)											// User-Abbruch
		{
		return -3;
		}


	for(iPos=0;iPos+1<sData.Count();iPos++)				// Suche doppelte Eintrge
		{
		pName =           sData.Get (iPos  );
		pInfo = (InfoTag*)sData.Data(iPos  );
		pNext = (InfoTag*)sData.Data(iPos+1);

		if(pNext->uHost!=pInfo->uHost  )continue;
		if(stricmp(pName,sData[iPos+1]))continue;

		sData.Del(iPos+1);
		iSave=1;
		}


	if(uSizeNew && uSize!=uSizeNew)						// Neue Puffergre
		{
		uSizeNew = uSize;
		iSave    = 1;
		}

	if(pHostCheck)										// Den Hostnamen berprfen
		{
		if(iIsTarget==0 && stricmp(cHost,pHostCheck))	// Neue Liste erzeugen bei anderem Quell-Host
			{
			   iRet = CreateList(pFileList,pSource,cPath,cMinus,uSize);
			if(iRet<0)return iRet;
			return Synch(pFileList,pSource,0,0,uSizeNew);
			}
		else if(iIsTarget!=0)
			{
			for(iPos=1;;iPos++)							// Prfen ob der Ziel-Host in der Liste ist
				{
				if(iPos>=10)return -2;
				if(stricmp(cHost+64*iPos,pHostCheck)==0)break;
				}
			}
		}


//******************** Dateien kopieren am Quell-Host *************************


	if(!stricmp(m_cHostName,cHost))
		{
		for(iPos=0;iPos<sData.Count();iPos++)			// Suche entfernte Dateien
			{
			pName = sData.Get (iPos);

			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

			   pInfo = (InfoTag*)sData.Data(iPos);
			if(pInfo->uHost!=0)
				{
				sData.Del(iPos);
				iSave=1;
				iPos--;
				continue;
				}

			strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);

			if(ExistFile(cBase)==1)
				{
				while(iPos+1<sData.Count())
					{
					if(stricmp(pName,sData[iPos+1]))break;
					iPos++;
					}

				continue;
				}

			while(iPos+1<sData.Count() && !stricmp(pName,sData[iPos+1]))
				{
				sData.Del(iPos+1);
				}

														// Datei aus dem Puffer entfernen
			strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);
			SS_CALLBACK(SSY_DELETE,cPath);
			RemoveFile(cPath);
			sData.Del(iPos);
			iSave=1;
			iPos--;
			}


		for(iPos=0;iPos<sData.Count();iPos++)			// Gleiche aus dem Puffer entferen
			{
			pName = sData.Get (iPos);

			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

			   pInfo = (InfoTag*)sData.Data(iPos);
			if(pInfo->uHost!=0)continue;

			memset(cBits,0,sizeof(cBits));

			for(iOk=1,iOff=1;iPos+iOff<sData.Count();iOff++)
				{
				if(stricmp(pName,sData[iPos+iOff]))break;
				pNext = (InfoTag*)sData.Data(iPos+iOff);

				if(pInfo->uChkSum==pNext->uChkSum)
				if(pInfo->uLength==pNext->uLength)
					{
					if(pNext->uHost<sizeof(cBits))
						{
						cBits[pNext->uHost]=1;
						}

					continue;
					}

				iOk = 0;
				break;
				}

			if(iOk)
			for(iOff=1;iOff<10;iOff++)					// Braucht ein anderer Host die Datei
				{
				if(cBits[iOff   ]   )continue;
				if(cHost[iOff*64]==0)continue;
				iOk=0;
				break;
				}


			strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);

			if(iOk && ExistFile(cPath)==1)				// Datei aus dem Puffer entfernen
				{
				SS_CALLBACK(SSY_DELETE,cPath);
				RemoveFile(cPath);
				}
			}


		SS_CALLBACK(SSY_MESSAGE,"Dateien suchen.");
		cBase[iNum]=0;
		sFind.Read(cBase,CFF_FIND_SUBDIR|CFF_FIND_ALL,cPlus,cMinus);

		for(iPos=0;iPos<sFind.Count();iPos++)			// Suche genderte original Dateien
			{
			pName = sFind.Name(iPos,2);
			iAt   = sData.FindFast(pName,0,-1,3);

			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

			if(iAt<0)									// Neue Datei gefunden
				{
				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);

				sInfo.uHost		= 0;
				sInfo.uLength	= sFind.Len (iPos);
				sInfo.uTime		= sFind.Time(iPos);
				sInfo.uChkSum	= GetChecksum(cBase);

				if(!sInfo.uChkSum)continue;

				sData.Add(pName,CSC_END,&sInfo);
				iSave=1;
				continue;
				}


			pInfo = (InfoTag*)sData.Data(iAt);

			if((int)pInfo->uTime  ==sFind.Time(iPos))	// Wurde die originale Datei gendert
			if((int)pInfo->uLength==sFind.Len (iPos))
				{
				continue;
				}

			strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
			sInfo.uLength	= sFind.Len (iPos);
			sInfo.uTime		= sFind.Time(iPos);
			sInfo.uChkSum	= GetChecksum(cBase);

			if(sInfo.uChkSum)
				{
				pInfo->uTime	= sInfo.uTime;
				pInfo->uLength	= sInfo.uLength;
				pInfo->uChkSum	= sInfo.uChkSum;
				iSave			= 1;
				}
			}



		SS_CALLBACK(SSY_MESSAGE,"Puffer durchsuchen.");
		cPath[iLen]=0;
		sFind.Read(cPath,CFF_FIND_SUBDIR|CFF_FIND_ALL,cPlus,cMinus);


		for(iPos=0;iPos<sFind.Count();iPos++)			// berflssige Dateien aus Puffer entfernen
			{
			pName = sFind.Name(iPos,2);
			iAt   = sData.FindFast(pName,0,-1,3);

			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

			if(iAt>=0)									// Existiert die orignal Datei zur Pufferdatei
				{
				pInfo = (InfoTag*)sData.Data(iAt);
				memset(cBits,0,sizeof(cBits));

				for(iOk=0,iOff=1;iAt+iOff<sData.Count();iOff++)
					{
					pNext = (InfoTag*)sData.Data(iAt+iOff);

					if(pNext->uHost<=pInfo->uHost)break;

					if(pNext->uChkSum==pInfo->uChkSum)
					if(pNext->uLength==pInfo->uLength)
					if(pNext->uHost<sizeof(cBits))
						{
						cBits[pNext->uHost]=1;
						}
					}

				for(iOk=1,iOff=1;iOff<10;iOff++)
					{
					if(cHost[iOff*64]==0)continue;
					if(cBits[iOff   ]!=0)continue;
					iOk=0;
					break;
					}

				if(!iOk)continue;
				}

			strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);
			SS_CALLBACK(SSY_DELETE,cPath);
			RemoveFile(cPath);
			sFind.Del(iPos);
			iPos--;
			}


		iRet  = 0;
		uByte = 0;

		for(iPos=0;iPos<sFind.Count();iPos++)			// Puffergre ermitteln
			{
			uByte += sFind.Len(iPos);
			}



		for(iPos=0;iPos<sData.Count();iPos++)			// Puffer fllen
			{
			if(uByte>=uSize)break;

			   pName =           sData.Get (iPos);
			   pInfo = (InfoTag*)sData.Data(iPos);
			if(pInfo->uHost!=0)continue;

			memset(cBits,0,sizeof(cBits));

			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

			for(iOff=1;iPos+iOff<sData.Count();iOff++)	// Existiert eine genderte Datei am Ziel
				{
				SS_CALLBACK(SSY_STOP,pName);

				if(m_iStop)return -3;

				if(stricmp(pName,sData[iPos+iOff]))break;
				pNext = (InfoTag*)sData.Data(iPos+iOff);

				if(pNext->uHost<sizeof(cBits))
					{
					cBits[pNext->uHost]=iOff;
					}

				if(pInfo->uChkSum==pNext->uChkSum)
				if(pInfo->uLength==pNext->uLength)
					{
					continue;
					}


				strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);
				iOk = sFind.Find(cPath);

				if(iOk>=0)continue;						// Ist die Datei schon im Puffer

				if(uByte+pInfo->uLength>uSize)			// Wird der Puffer zu gro
					{
					continue;
					}

				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);

				if(ExistFile(cPath)==1)continue;

				SS_CALLBACK(SSY_COPY,cPath);

				GetFilePath(cTemp,cPath,sizeof(cTemp));	// Datei in den Puffer einfgen
				CreatePath (cTemp);

				if(CopyFile(cBase,cPath,FALSE))
					{
					memset(cBits,0xFF,sizeof(cBits));
					uByte += pInfo->uLength;
					iRet++;
					}
				}

			for(iOff=1;iOff<10;iOff++)					// Neue Dateien einfgen
				{
				SS_CALLBACK(SSY_STOP,pName);

				if(m_iStop)return -3;

				if(cBits[iOff   ]   )continue;
				if(cHost[iOff*64]==0)continue;

				if(uByte+pInfo->uLength>uSize)			// Wird der Puffer zu gro
					{
					break;
					}

				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);

				if(ExistFile(cPath)==1)break;

				SS_CALLBACK(SSY_COPY,cPath);

				GetFilePath(cTemp,cPath,sizeof(cTemp));	// Datei in den Puffer einfgen
				CreatePath (cTemp);

				if(CopyFile(cBase,cPath,FALSE))
					{
					uByte += pInfo->uLength;
					iRet++;
					}
				}
			}

		if(iSave)
			{
			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

				pFile = fopen(pFileList,"wb");
			if(!pFile)return -1;

			fprintf(pFile,"PLUS  \"%s\"\n",cPlus);
			fprintf(pFile,"MINUS \"%s\"\n",cMinus);
			fprintf(pFile,"HOST0 \"%s\"\n",m_cHostName);

			for(iPos=1;iPos<10;iPos++)					// Hostnamen speichern
				{
				if(!cHost[64*iPos])continue;
				fprintf(pFile,"HOST%i \"%s\"\n",iPos,cHost+64*iPos);
				}

			fprintf(pFile,"SIZE  %u\n"    ,uSize);


			for(iPos=0;iPos<sData.Count();iPos++)		// Dateiliste speichern
				{
				pName = sData.Get(iPos);
				pInfo = (InfoTag*)sData.Data(iPos);
				fprintf(pFile,"%i\t0x%08X 0x%08X 0x%08X\t\"%s\"\n",pInfo->uHost,pInfo->uLength,pInfo->uTime,pInfo->uChkSum,pName);
				}

			fclose(pFile);
			}

		cPath[iLen]=0;
		sFind.Read(cPath,CFF_FIND_SUBDIR|CFF_FIND_ADDDIR|CFF_FIND_ALL,cPlus,cMinus);
		sFind.Sort(CFF_SORT_PATH,CFF_SORT_NODIR);

		for(iPos=0;iPos<sFind.Count();iPos++)			// Lsche leere Verzeichnisse
			{
			if(!sFind.IsDir(iPos))continue;

			if(iPos+1>=sFind.Count() || sFind.IsDir(iPos+1))
				{
				pName = sFind.Name(iPos,1);
				SS_CALLBACK(SSY_DELETE,pName);
				RemoveDirectory(pName);
				}
			}


		return iRet;
		}


//******************** Backup am Zielrechner **********************************


	for(uHost=1;uHost<10;uHost++)						// Suche den Hostnamen
		{
		if(!stricmp(cHost+uHost*64,m_cHostName))break;
		}

	if(uHost>=10)										// Leerplatz suchen
		{
		for(uHost=1;uHost<10;uHost++)
			{
			if(!cHost[uHost*64])break;
			}

		if(uHost>=10)return -1;

		strlcpy(cHost+uHost*64,m_cHostName,64);
		iSave=1;
		}

	iRet  = 0;
	iNull = 0;

	for(iPos=0;iPos<sData.Count();iPos++)				// Zhle die Quelldateien
		{
		   pInfo = (InfoTag*)sData.Data(iPos);
		if(pInfo->uHost==0)iNull++;
		}


	SS_CALLBACK(SSY_MESSAGE,"Dateien suchen.");
	cBase[iNum]=0;
	sFind.Read(cBase,CFF_FIND_SUBDIR|CFF_FIND_ALL,cPlus,cMinus);

	for(iPos=0;iPos<sFind.Count();iPos++)				// Suche genderte original Dateien
		{
		pName = sFind.Name(iPos,2);
		iAt   = sData.FindFast(pName,0,-1,3);

		SS_CALLBACK(SSY_STOP,pName);

		if(m_iStop)return -3;

		if(iAt<0)										// berflssige Dateien im Ziel entferen
			{
			if(iNull<=0)continue;

			strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
			RemoveFile(cBase);

			continue;
			}

		pInfo = (InfoTag*)sData.Data(iAt);

		if(pInfo->uHost!=0)								// Gibt es keinen Quelleintrag ?
			{
			for(iOk=0,iOff=0;iAt+iOff<sData.Count();iOff++)
				{										// Suche Zieleintrag in der Liste
				pNext = (InfoTag*)sData.Data(iAt+iOff);

				if(pNext->uHost>uHost || iOff>10)break;
				if(pNext->uHost<=pInfo->uHost && iOff)break;
				if(pNext->uHost==uHost){iOk=1;break;}
				}

			if(iOk==0 && iNull>0)						// Zieldatei lschen
				{
				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				RemoveFile(cBase);
				continue;
				}

			if(iOk==0 && iNull==0)						// Neuen Zieleintrag einfgen
				{
				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				sInfo.uHost		= uHost;
				sInfo.uLength	= sFind.Len (iPos);
				sInfo.uTime		= sFind.Time(iPos);
				sInfo.uChkSum	= GetChecksum(cBase);

				if(!sInfo.uChkSum)continue;

				sData.Add(pName,iAt+iOff,&sInfo);
				iSave=1;
				continue;
				}

			if(iNull>0)									// Zieldatei lschen
				{
				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				RemoveFile(cBase);
				sData.Del(iAt+iOff);
				iSave=1;
				continue;
				}


			sInfo.uLength	= sFind.Len (iPos);
			sInfo.uTime		= sFind.Time(iPos);

														// Hat sich die Datei im Zeilordner verndert
			if(sInfo.uTime!=pNext->uTime || sInfo.uLength!=pNext->uLength)
				{
				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				sInfo.uChkSum	= GetChecksum(cBase);

				if(sInfo.uChkSum)
					{
					pNext->uChkSum = sInfo.uChkSum;
					pNext->uLength = sInfo.uLength;
					pNext->uTime   = sInfo.uTime;
					iSave=1;
					}
				}


			continue;
			}


		for(iOk=0,iOff=1;iAt+iOff<sData.Count();iOff++)	// Suche Zieleintrag in der Liste
			{
			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

			   pNext = (InfoTag*)sData.Data(iAt+iOff);
			if(pNext->uHost<=pInfo->uHost || pNext->uHost>uHost || iOff>10)
				{
				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				sInfo.uHost		= uHost;				// Datei aus Zeilordner ist nicht in der Liste
				sInfo.uLength	= sFind.Len (iPos);
				sInfo.uTime		= sFind.Time(iPos);
				sInfo.uChkSum	= GetChecksum(cBase);

				if(!sInfo.uChkSum)break;

				sData.Add(pName,iAt+iOff,&sInfo);		// Neue Zeileintrag
				iSave=1;
				iOk=1;
				break;
				}

			if(pNext->uHost!=uHost)continue;

			sInfo.uLength	= sFind.Len (iPos);
			sInfo.uTime		= sFind.Time(iPos);

														// Hat sich die Datei im Zeilordner verndert
			if(sInfo.uTime!=pNext->uTime || sInfo.uLength!=pNext->uLength)
				{
				strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);
				sInfo.uChkSum	= GetChecksum(cBase);

				if(sInfo.uChkSum)
					{
					pNext->uChkSum = sInfo.uChkSum;
					pNext->uLength = sInfo.uLength;
					pNext->uTime   = sInfo.uTime;
					iSave=1;
					}
				}

			break;
			}
		}



	for(iPos=0;iPos<sData.Count();iPos++)				// Synchronisiere die Dateien
		{
		pName = sData.Get (iPos);

		SS_CALLBACK(SSY_STOP,pName);

		if(m_iStop)return -3;

		   pInfo = (InfoTag*)sData.Data(iPos);
		if(pInfo->uHost!=0)continue;


		for(iOk=0,iOff=1;iPos+iOff<sData.Count();iOff++)// Suche Zieleintrag in der Liste
			{
			SS_CALLBACK(SSY_STOP,pName);

			if(m_iStop)return -3;

			   pNext = (InfoTag*)sData.Data(iPos+iOff);
			if(pNext->uHost<=pInfo->uHost || pNext->uHost>uHost || iOff>10)
				{
				break;
				}

			if(pNext->uHost!=uHost)continue;

			iOk=1;

			if(pInfo->uChkSum==pNext->uChkSum)
			if(pInfo->uLength==pNext->uLength)break;

			strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);
			strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);

			if(ExistFile(cPath)!=1)break;

			SS_CALLBACK(SSY_COPY,cBase);

			GetFilePath(cTemp,cBase,sizeof(cTemp));		// Datei aus dem Puffer kopieren
			CreatePath (cTemp);

			if(CopyFile(cPath,cBase,FALSE))
				{
				pNext->uChkSum = pInfo->uChkSum;
				pNext->uLength = pInfo->uLength;
				iSave = 1;
				iRet++;
				}

			break;
			}

		if(!iOk)
			{
			strlcpy(cPath+iLen,pName,sizeof(cPath)-iLen);
			strlcpy(cBase+iNum,pName,sizeof(cBase)-iNum);

			if(ExistFile(cPath)!=1)continue;

			SS_CALLBACK(SSY_COPY,cBase);

			GetFilePath(cTemp,cBase,sizeof(cTemp));		// Datei aus dem Puffer kopieren
			CreatePath (cTemp);


			if(CopyFile(cPath,cBase,FALSE))
				{
				sInfo       = *pInfo;
				sInfo.uHost =  uHost;
				sData.Add(pName,iPos+iOff,&sInfo);
				iSave=1;
				iRet++;
				}
			}
		}


//******************** Liste speichern ****************************************

	if(iSave)
		{
		SS_CALLBACK(SSY_STOP,pFileList);

		if(m_iStop)return -3;

			pFile = fopen(pFileList,"wb");
		if(!pFile)return -1;

		fprintf(pFile,"PLUS  \"%s\"\n",cPlus);
		fprintf(pFile,"MINUS \"%s\"\n",cMinus);

		for(iPos=0;iPos<10;iPos++)						// Hostnamen speichern
			{
			if(!cHost[64*iPos])continue;
			fprintf(pFile,"HOST%i \"%s\"\n",iPos,cHost+64*iPos);
			}

		fprintf(pFile,"SIZE  %u\n"    ,uSize);


		for(iPos=0;iPos<sData.Count();iPos++)			// Dateiliste speichern
			{
			pName = sData.Get(iPos);
			pInfo = (InfoTag*)sData.Data(iPos);
			fprintf(pFile,"%i\t0x%08X 0x%08X 0x%08X\t\"%s\"\n",pInfo->uHost,pInfo->uLength,pInfo->uTime,pInfo->uChkSum,pName);
			}

		fclose(pFile);
		}



return iRet;
}


//*****************************************************************************
//*
//*		SetCallback
//*
//*****************************************************************************
//	Stellt die Callbach Funktion ein
//	pProc		: Ist ein Zeiger auf die Funktion
//	pParam		: Der bergabeparameter fr die Funktion
void CSizeSynch::SetCallback(void *pParam,SizeSynchProc pProc)
{

	m_pCallback	 = pProc;
	m_pCallParam = pParam;

}
